"""Classes for implementing damage curves and economic damage

   Duncan Gray, Ole Nielsen, Jane Sexton, Nick Bartzis
   Geoscience Australia, 2006
"""

from math import sqrt
from Scientific.Functions.Interpolation import InterpolatingFunction
from Numeric import array


try:  
    import kinds  
except ImportError:  
    # Hand-built mockup of the things we need from the kinds package, since it  
    # was recently removed from the standard Numeric distro.  Some users may  
    # not have it by default.  
    class _bunch:  
        pass  
         
    class _kinds(_bunch):  
        default_float_kind = _bunch()  
        default_float_kind.MIN = 2.2250738585072014e-308  #smallest +ve number
        default_float_kind.MAX = 1.7976931348623157e+308  
     
    kinds = _kinds()
    

from pyvolution.data_manager import Exposure_csv
from pyvolution.util import file_function
from geospatial_data.geospatial_data import ensure_absolute
from utilities.numerical_tools import INF
from anuga_config import epsilon

def inundation_damage(sww_file, exposure_file_in,
                      exposure_file_out=None,
                      overwrite=False, verbose=True,
                                 use_cache = True):
    """
    """

    csv = Exposure_csv(exposure_file_in)
    geospatial = csv.get_location()
    max_depths, max_momentums = calc_max_depth_and_momentum(sww_file,
                                                          geospatial,
                                                          verbose=verbose,
                                                          use_cache=use_cache)
    #Do the two damage curves
    # Ole probably knows if ther is good code for this lying around
    #Do the prob of struct collapse
    #Do economics
    # Save info back to csv file

    
def add_depth_and_momentum2csv(sww_file, exposure_file_in,
                      exposure_file_out=None,
                      overwrite=False, verbose=True,
                                 use_cache = True):
    """
    """

    csv = Exposure_csv(exposure_file_in)
    geospatial = csv.get_location()
    max_depths, max_momentums = calc_max_depth_and_momentum(sww_file,
                                                          geospatial,
                                                          verbose=verbose,
                                                          use_cache=use_cache)
    csv.set_column("MAX INUNDATION DEPTH (m)",max_depths, overwrite=overwrite)
    csv.set_column("MOMENTUM (m^2/s) ",max_momentums, overwrite=overwrite)
    csv.save(exposure_file_out)
    
def calc_max_depth_and_momentum(sww_file, points,
                                ground_floor_height=0.0,
                                verbose=True,
                                 use_cache = True):
    """
    Calculate the maximum inundation height above ground floor (mihagf) for a list
    of locations.

    The mihagf value is in the range -ground_floor_height to overflow errors.
    """
    quantities =  ['stage', 'elevation', 'xmomentum', 'ymomentum']
    points = ensure_absolute(points)
    point_count = len(points)
    callable_sww = file_function(sww_file,
                                 quantities=quantities,
                                 interpolation_points=points,
                                 verbose=verbose,
                                 use_cache=use_cache)
    # initialise the max lists
    max_depths = [ground_floor_height]*point_count
    max_momentums = [ground_floor_height]*point_count
    for point_i, point in enumerate(points):
        for time in callable_sww.get_time():
            quantities = callable_sww(time,point_i)
            #print "quantities", quantities
            
            w = quantities[0]
            z = quantities[1]
            uh = quantities[2]
            vh = quantities[3]

            # This can easily give -ve values,
            # but the array is initialised to 0.0,
            # so 0.0 will be the minimum value.
            depth = w - z - ground_floor_height
              
            if depth > max_depths[point_i]:
                max_depths[point_i] = depth
            if w == INF or z == INF or uh == INF or vh == INF:
                continue
            momentum = sqrt(uh*uh + vh*vh)
            if momentum > max_momentums[point_i]:
                max_momentums[point_i] = momentum
    return max_depths, max_momentums

class EventDamageModel:
    """
    Initially I'm doing this as a class to group all of these functions/methods.


    """
    double_brick_damage_curve = array([[-kinds.default_float_kind.MAX, 0.0],
                                    [0.0-epsilon, 0.0],
                                    [0.0,1.6],
                                    [2.5,70.0],
                                    [kinds.default_float_kind.MAX,70]])
     
    def __init__(self,max_depths, shore_distances, walls):
        self.max_depths = max_depths
        self.shore_distances = shore_distances
        self.walls = walls
        assert len(self.max_depths) == len(self.shore_distances)
        self.structure_count = len(self.max_depths)
        structure_damage_curve = InterpolatingFunction((array([0,1]),),array([0,10]))

    def calc_damage_percentages(self):

        # the data being created
        percent_struct_damage = [0.0]*self.structure_count
        percent_contents_damage = [0.0]*self.structure_count
        #for i_building,

        for i,max_height,shore_distance,wall in map(None,
                                                    range(self.structure_count),
                                                    self.max_depths,
                                                    self.shore_distances,
                                                    self.walls):
            print "i",i
            print "max_height",max_height
            print "shore_distance",shore_distance
           
        
#############################################################################
if __name__ == "__main__":
    from Scientific.Functions.Interpolation import InterpolatingFunction
    from Numeric import array, ravel
    a = array([[0,0],[1,10]])
    c = array([0,1])

    axis = ravel(a[:,0:1])
    values = ravel(a[:,1:])
    
    #axis = array([0,1])
    #values = array([0,10])
    print "axis",axis
    print "values",values 
    i = InterpolatingFunction((axis,), values)
    print "value",i(0.5)

    
    i = InterpolatingFunction((array([0,1]),),array([0,10]))
    print "value",i(0.5)

        
