#!/usr/bin/env python
#

import unittest
import tempfile
import os
import time
import csv

from damage.inundation_damage import *
from geospatial_data.geospatial_data import Geospatial_data
from pmesh.mesh import Mesh
from coordinate_transforms.geo_reference import Geo_reference
from pyvolution.shallow_water import Domain, Transmissive_boundary
from utilities.numerical_tools import mean
from data_manager import get_dataobject

from Numeric import zeros, Float
        
class Test_inundation_damage(unittest.TestCase):
    def setUp(self):
        #print "****set up****"
        # Create an sww file
        
        # Set up an sww that has a geo ref.
        # have it cover an area in Australia.  'gong maybe
        #Don't have many triangles though!
        
        #Site Name:    GDA-MGA: (UTM with GRS80 ellipsoid) 
        #Zone:   56    
        #Easting:  222908.705  Northing: 6233785.284 
        #Latitude:   -34  0 ' 0.00000 ''  Longitude: 150  0 ' 0.00000 '' 
        #Grid Convergence:  -1  40 ' 43.13 ''  Point Scale: 1.00054660

        #geo-ref
        #Zone:   56    
        #Easting:  220000  Northing: 6230000 


        #have  a big area covered.

        mesh_file = tempfile.mktemp(".tsh")
        
        points_lat_long = [[-33,152],[-35,152],[-35,150],[-33,150]]
       
        spat = Geospatial_data(data_points=points_lat_long,
                               points_are_lats_longs=True)
        points_ab = spat.get_data_points( absolute = True)

        #FIXME - put this back when it is working
        #geo =  Geo_reference(56,400000,6000000)
        #spat.set_geo_reference(geo)

        m = Mesh()
        m.add_vertices(spat)
        m.autoSegment()
        m.generate_mesh(verbose=False)
        m.export_mesh_file(mesh_file)

        
        #Create shallow water domain
        domain = Domain(mesh_file)

        os.remove(mesh_file)
        
        domain.default_order=2
        domain.beta_h = 0


        #Set some field values
        #domain.set_quantity('stage', 1.0)
        domain.set_quantity('elevation', -0.5)
        domain.set_quantity('friction', 0.03)


        ######################
        # Boundary conditions
        B = Transmissive_boundary(domain)
        domain.set_boundary( {'exterior': B})


        ######################
        #Initial condition - with jumps

        bed = domain.quantities['elevation'].vertex_values
        stage = zeros(bed.shape, Float)

        h = 0.3
        for i in range(stage.shape[0]):
            if i % 2 == 0:
                stage[i,:] = bed[i,:] + h
            else:
                stage[i,:] = bed[i,:]

        domain.set_quantity('stage', stage)
        domain.set_quantity('xmomentum', stage*22.0)
        domain.set_quantity('ymomentum', stage*55.0)

        domain.distribute_to_vertices_and_edges()


        self.domain = domain

        C = domain.get_vertex_coordinates()
        self.X = C[:,0:6:2].copy()
        self.Y = C[:,1:6:2].copy()

        self.F = bed

        
        #sww_file = tempfile.mktemp("")
        self.domain.filename = 'datatest' + str(time.time())
        #self.domain.filename = sww_file
        #print "self.domain.filename",self.domain.filename 
        self.domain.format = 'sww'
        self.domain.smooth = True
        self.domain.reduction = mean

        sww = get_dataobject(self.domain)
        sww.store_connectivity()
        sww.store_timestep(['stage', 'xmomentum', 'ymomentum'])
        self.domain.time = 2.
        sww.store_timestep(['stage', 'xmomentum', 'ymomentum'])
        self.sww = sww # so it can be deleted
        
        #Create a csv file
        self.csv_file = tempfile.mktemp(".csv")
        fd = open(self.csv_file,'wb')
        writer = csv.writer(fd)
        writer.writerow(['LONGITUDE','LATITUDE','STR_VALUE','C_VALUE','ROOF_TYPE','WALLS', 'SHORE_DIST'])
        writer.writerow(['151.5','-34','199770','130000','Metal','Timber',20])
        writer.writerow(['151','-34.5','150000','76000','Metal','Double Brick',200])

        
    def tearDown(self):
        #print "***** tearDown  ********"
        os.remove(self.sww.filename)
        os.remove(self.csv_file)

    
    def test_inundation_damage(self):
        sww_file = self.domain.filename + "." + self.domain.format
        #print "sww_file",sww_file 
        inundation_damage(sww_file, self.csv_file, verbose=False)
        
    def ztest_add_depth_and_momentum2csv(self):
        sww_file = self.domain.filename + "." + self.domain.format
        #print "sww_file",sww_file
        
        out_csv = tempfile.mktemp(".csv")
        print "out_csv",out_csv 
        add_depth_and_momentum2csv(sww_file, self.csv_file,
                                   out_csv, verbose=False)
        
    def test_calc_damage_percentages(self):
        max_depths = [-0.3, 0.0, 1.0]
        shore_distances = [100, 200, 300]
        walls = ['Double Brick','Timber','Brick Veneer']

        edm = EventDamageModel(max_depths, shore_distances, walls)
        edm.calc_damage_percentages()
        
#-------------------------------------------------------------
if __name__ == "__main__":
    #suite = unittest.makeSuite(Test_inundation_damage,'test_add_depth_and_momentum2csv')
    #suite = unittest.makeSuite(Test_inundation_damage,'test_inundation_damage')
    suite = unittest.makeSuite(Test_inundation_damage,'test')
    runner = unittest.TextTestRunner()
    runner.run(suite)

